﻿using System;
using CoreGraphics;
using UIKit;
using Foundation;

namespace OneMemo
{
	public partial class ViewController : UIViewController
	{
		enum SelectLR { ZERO, SELECT_LEFT, SELECT_RIGHT };
		SelectLR _currentSLR = SelectLR.ZERO;
		NSObject _showObserver;
		NSObject _hideObserver;

		protected ViewController(IntPtr handle) : base(handle)
		{
		}

		public override void ViewDidLoad()
		{
			base.ViewDidLoad();

			UIToolbar toolBar = new UIToolbar(new CGRect(0, 0, 320, 40));
			toolBar.BarStyle = UIBarStyle.Default;
			toolBar.SizeToFit();
			UIBarButtonItem spacer = new UIBarButtonItem(UIBarButtonSystemItem.FlexibleSpace, this, null);
			UIBarButtonItem commitBtn = new UIBarButtonItem(UIBarButtonSystemItem.Done, delegate { _textView.ResignFirstResponder(); });
			UIBarButtonItem leftBtn = new UIBarButtonItem("←", UIBarButtonItemStyle.Plain, delegate { LeftCaret(); });
			UIBarButtonItem rightBtn = new UIBarButtonItem("→", UIBarButtonItemStyle.Plain, delegate { RightCaret(); });
			UIBarButtonItem leftSelectBtn = new UIBarButtonItem("⇦", UIBarButtonItemStyle.Plain, delegate { LeftSelectCaret(); });
			UIBarButtonItem rightSelectBtn = new UIBarButtonItem("⇨", UIBarButtonItemStyle.Plain, delegate { RightSelectCaret(); });
			UIBarButtonItem cutBtn = new UIBarButtonItem("cut", UIBarButtonItemStyle.Plain, delegate { CutCaret(); });
			UIBarButtonItem copyBtn = new UIBarButtonItem("copy", UIBarButtonItemStyle.Plain, delegate { CopyCaret(); });
			UIBarButtonItem pasteBtn = new UIBarButtonItem("paste", UIBarButtonItemStyle.Plain, delegate { PasteCaret(); });
			UIBarButtonItem[] toolBarItems = new UIBarButtonItem[] { leftBtn, rightBtn, leftSelectBtn, rightSelectBtn, cutBtn, copyBtn, pasteBtn, spacer, commitBtn };
			toolBar.SetItems(toolBarItems, true);
			_textView.InputAccessoryView = toolBar;
			_textView.Text = AppDelegate._text;
			_textView.Changed += (sender, e) => { AppDelegate._text = new NSString(_textView.Text); };
		}

		public override void DidReceiveMemoryWarning()
		{
			base.DidReceiveMemoryWarning();
		}

		private void LeftCaret()
		{
			UITextRange range = _textView.SelectedTextRange;
			UITextPosition position = _textView.GetPosition(range.Start, -1);
			if (position != null)
				_textView.SelectedTextRange = _textView.GetTextRange(position, position);
			_currentSLR = SelectLR.ZERO;
		}

		private void RightCaret()
		{
			UITextRange range = _textView.SelectedTextRange;
			UITextPosition position = _textView.GetPosition(range.Start, 1);
			if (position != null)
				_textView.SelectedTextRange = _textView.GetTextRange(position, position);
			_currentSLR = SelectLR.ZERO;
		}

		private void LeftSelectCaret()
		{
			UITextRange range = _textView.SelectedTextRange;
			UITextPosition position;
			if (range.IsEmpty) _currentSLR = SelectLR.ZERO;
			switch (_currentSLR)
			{
				case SelectLR.ZERO:
				case SelectLR.SELECT_LEFT:
					_currentSLR = SelectLR.SELECT_LEFT;
					position = _textView.GetPosition(range.Start, -1);
					if (position != null)
						_textView.SelectedTextRange = _textView.GetTextRange(position, range.End);
					break;
				case SelectLR.SELECT_RIGHT:
					position = _textView.GetPosition(range.End, -1);
					if (position != null)
						_textView.SelectedTextRange = _textView.GetTextRange(range.Start, position);
					break;
			}
		}

		private void RightSelectCaret()
		{
			UITextRange range = _textView.SelectedTextRange;
			UITextPosition position;
			if (range.IsEmpty) _currentSLR = SelectLR.ZERO;
			switch (_currentSLR)
			{
				case SelectLR.ZERO:
				case SelectLR.SELECT_RIGHT:
					_currentSLR = SelectLR.SELECT_RIGHT;
					position = _textView.GetPosition(range.End, 1);
					if (position != null)
						_textView.SelectedTextRange = _textView.GetTextRange(range.Start, position);
					break;
				case SelectLR.SELECT_LEFT:
					position = _textView.GetPosition(range.Start, 1);
					if (position != null)
						_textView.SelectedTextRange = _textView.GetTextRange(position, range.End);
					break;
			}
		}

		private void CutCaret()
		{
			UIPasteboard clipboard = UIPasteboard.General;
			UITextRange range = _textView.SelectedTextRange;
			int index = (int)_textView.GetOffsetFromPosition(_textView.BeginningOfDocument, range.Start);
			int count = (int)_textView.GetOffsetFromPosition(range.Start, range.End);
			clipboard.String = _textView.TextInRange(range);
			_textView.Text = _textView.Text.Remove(index, count);
			_textView.SelectedTextRange = _textView.GetTextRange(range.Start, range.Start);
			AppDelegate._text = new NSString(_textView.Text);
		}

		private void CopyCaret()
		{
			UIPasteboard clipboard = UIPasteboard.General;
			UITextRange range = _textView.SelectedTextRange;
			clipboard.String = _textView.TextInRange(range);
		}

		private void PasteCaret()
		{
			UIPasteboard clipboard = UIPasteboard.General;
			UITextRange range = _textView.SelectedTextRange;
			int index = (int)_textView.GetOffsetFromPosition(_textView.BeginningOfDocument, range.Start);
			int count = (int)_textView.GetOffsetFromPosition(range.Start, range.End);
			_textView.Text = _textView.Text.Remove(index, count);
			_textView.Text = _textView.Text.Insert(index, clipboard.String);
			_textView.SelectedTextRange = _textView.GetTextRange(range.Start, range.Start);
			AppDelegate._text = new NSString(_textView.Text);
		}

		public override void ViewWillAppear(bool animated)
		{
			base.ViewWillAppear(animated);

			_showObserver = NSNotificationCenter.DefaultCenter.AddObserver(
				UIKeyboard.WillShowNotification, n =>
				{
					var v = ((NSNumber)n.UserInfo[UIKeyboard.AnimationDurationUserInfoKey]).DoubleValue;
					var h = ((NSValue)n.UserInfo[UIKeyboard.FrameBeginUserInfoKey]).CGRectValue.Height;
					var rect = _textView.Frame;
					rect.Height = View.Bounds.Height - 20 - h;
					UIView.Animate(v,
			          	() =>
						{
							_textView.Frame = rect;
						});
				});
			_hideObserver = NSNotificationCenter.DefaultCenter.AddObserver(
				UIKeyboard.WillHideNotification, n =>
				{
					var v = ((NSNumber)n.UserInfo[UIKeyboard.AnimationDurationUserInfoKey]).DoubleValue;
					var h = ((NSValue)n.UserInfo[UIKeyboard.FrameBeginUserInfoKey]).CGRectValue.Height;
					var rect = _textView.Frame;
					UIView.Animate(v,
						() =>
						{
							rect.Height = View.Bounds.Height - 20;
							_textView.Frame = rect;
						});
				});
		}

		public override void ViewWillDisappear(bool animated)
		{
			base.ViewWillDisappear(animated);
			NSNotificationCenter.DefaultCenter.RemoveObservers(new[] { _showObserver, _hideObserver });
		}
	}
}
